	name    KCT
	title   Kansas City Tracker TSR
;
; Copyright 1998 (C), by Michael Radwick, N8XKZ
;
;  This program does satellite antenna control via the KCT TSR.
;  The program assumes the Yaesu 5400B controler interfaced
;  to the IBM-PC using LPT1 parallel port.  The interface card
;  was originally designed by Manfred Mornhinweg (XQ2FOD)
;

;
; parallel port interface:
; PC      | Bit | Corresponding | Pin No. |   I/O
; Address | No. |   Pin Name    |  DB-25  | Direction
;---------|-----|---------------|---------|-----------
;  Base   |  0  |      D0       |    2    |         
;         |  1  |      D1       |    3    |
;         |  2  |      D2       |    4    |
;         |  3  |      D3       |    5    |  Output
;         |  4  |      D4       |    6    |   Only
;         |  5  |      D5       |    7    |
;         |  6  |      D6       |    8    |
;         |  7  |      D7       |    9    |
;---------|-----|---------------|---------|-----------
; Base+1  |  0  |   Not Used    |   NA    |    NA
;         |  1  |   Not Used    |   NA    |    NA
;         |  2  |   Not Used    |   NA    |    NA
;         |  3  |   -Error      |   15    |   Input Only
;         |  4  |    SLCTD      |   13    |   Input Only
;         |  5  |      PE       |   12    |   Input Only
;         |  6  |     -ACK      |   10    |   Input Only
;         |  7  |     BUSY      |   11    |   Input Only
;---------|-----|---------------|---------|-----------
; Base+2  |  0  |    Strobe     |    1    | Input & Output
;         |  1  | Auto FD XT    |   14    | Input & Output
;         |  2  |    -Init      |   16    | Input & Output
;         |  3  |   SLCTD IN    |   17    | Input & Output
;         |  4  |  IRQ Enable   |   NA    |      NA
;         |  5  |   Not Used    |   NA    |      NA
;         |  6  |   Not Used    |   NA    |      NA
;         |  7  |   Not Used    |   NA    |      NA
;---------|-----|---------------|---------|-----------

cr              EQU     0Dh
lf              EQU     0Ah

LPT1            EQU     0378h
LPT2            EQU     0278h

DATAOUT         EQU     0          ; offset to 8255 port a
CTRLIN          EQU     1          ;   "       8255 port b
CTRLOUT         EQU     2          ;   "       8255 port c
CONTROL         EQU     3          ;   "       8255 control port

KCTINT          EQU     0063h           ;interrupt hook KCT TSR onto.

command         equ     80h             ;offset from start of
					;program segment prefix to
					;command line arguments.

_TEXT   segment word public 'CODE'

	ORG     100h                    ; this is a .COM file!

	assume  cs:_TEXT, ds:_TEXT, es:_TEXT, ss:_TEXT

	EXTRN getargs:PROC, xtoi:PROC, atoi:PROC

install proc    near

	mov     ah,9                    ; say hello
	mov     dx,offset hello
	int     21h

	call    use_lpt1                ; set default port

	mov     si,offset command
	cld
	lodsb                           ; are there any command options?
	or      al,al
	jz      skip_opt
	xor     ah,ah
	mov     bx,ax
	mov     byte ptr [bx+si],0      ; terminate command line.
	mov     di,si
	mov     si,offset optTable
	call    getargs                 ; process command line options.

skip_opt:
	mov     dx, offset isr          ; get address of service routine
	mov     ax, 2500h OR KCTINT     ; set KCT interrupt hook
	int     21h                     ; set up the interrupt vector

				; reserve memory and terminate.
	mov     dx,offset lastbyte      ; get last address used
	add     dx,15           ; round up
	mov     cl,4
	shr     dx,cl           ; convert to paragraphs
	mov     ax,3100h        ; TSR function code, good status
	int     21H             ; go TSR!

hello   db      cr,lf
	db      'FODTRK Antenna Tracker for InstantTrack. Ver 1.1',cr,lf
	db      'by Michael Radwick, N8XKZ, copyright 1998',cr,lf,'$'

install endp

enabled db      1               ; enabled=1, disabled=0
azim    dw      0               ; save target az and el
elev    dw      0
azOffset        dw      0
elOffset        dw      0

lptDataOut      dw      0
lptCtrlIn       dw      0
lptCtrlOut      dw      0
lptControl      dw      0

;******************************************************************
;  Here is the interrupt service routine.
;******************************************************************

	assume  ds:nothing, es:nothing, ss:nothing

isr:    jmp     dispatch

	ORG     isr + 10
	DB      "RotorDRV"      ; signature string
;
; dispatch - Entry point for int 0x63
;   Whenever a satellite tracking program wants to control the
;   antenna using a KCT compatible interface, it will call
;   int 0x63, which is routed to here.
;
;   This function figures out which command is being requested,
;   and jumps to the appropriate handler.  Each handler is
;   expected to exit via a 'iret'.
;
dispatch proc   near
	sti
	cmp     ah,0            ; dispatch according to function code
	je      readstat
	cmp     ah,3
	je      readpos
	cmp     ah,4
	je      settarg
	cmp     ah,9
	je      enable
	iret
dispatch endp

enable  proc    near
	mov     cs:enabled,al      ; save the enable/disable flag.
	iret
enable  endp

readstat proc   near
	mov     al,0            ; status is always good.
	iret
readstat endp

readpos proc    near
	mov     ax,cs:elev
	mov     cx,ax
	mov     bx,cs:azim
	mov     dx,bx
	iret
readpos endp

;******************************************************************
; Function 04: Set Target position
; --------------------------------
; This function is used by the TSR to tell the tracker TSR where to
; point the antennas.  It is called periodically.
;
;    At Entry: AH=4
;              CX - target ELEV value in degrees (-90 to 269, signed binary)
;              DX - target AZIM value in degrees (0-359, binary)
;              Note: If either value is invalid the entire command will
;                    be ignored.
;
;    At Exit:  No parameters are returned
;*******************************************************************
; 123456789
; +000 +000
;  az   el

settarg proc    near
	cmp     cs:enabled,0       ; don't do anything if disabled.
	jne     setAzEl
	jmp     st_exit

setAzEl:
	pushf                   ; save direction flag
	push    dx              ; save azimuth
	push    cx              ; save elevation
	mov     cs:azim,dx
	mov     cs:elev,cx

; we need to convert 0-359 to an 8bit number where
; 00h = 181 degrees, 40h = 270, 80h = 0, C0h = 90, and FFh = 180 degrees
;
; the equation for this is (((az+180)/7)*5)
;
	mov     ax,dx
	cmp     ax,180          ; adjust requested position so that
	jle     calcaz          ; azimuth 181-359 maps into hex 0-7F
	sub     ax,360
calcaz:
	add     ax,cs:azOffset  ; Add in the user's azimuth offset
	add     ax,180          ; add in constant 180.
				; Hence requested values 181 to 359
				; are converted to 1 to 179 and
				; values 0 to 180 are converted to
				; 180 to 360.
	cmp     ax,360          ; Warp the value around the
	jl      doAz            ; great circle.
	sub     ax,360
doAz:
	mov     bl,7            ;convert value to FODTRACK units.
	idiv    bl
	mov     bh,ah           ;save remainder
	mov     bl,5
	imul    bl

	mov     dx,cs:lptDataOut        ;Set Azimuth in FODTRK
	out     dx,al
	;**********************************************
	; NOP instructions are used to provide a simple
	; timing delay which allows the FODTRACK
	; hardware to settle.  On the strobes, there
	; must be enough time to meet the electronics'
	; "set-up and hold time".
	; The delays are good enough for a 40Mhz
	; PC, But I don't know about faster computers.
	;
	; TO DO: convert the NOP's to a subroutine call
	;        which will delay a variable amount of
	;        time which is a function of a command-line
	;        option.  This will allow the delay to
	;        be increased (if needed) by the user
	;        when the driver is loaded.
	;***********************************************
	nop
	nop

	mov     dx,cs:lptCtrlOut
	mov     al,02h                  ;make sure az is selected.
	out     dx,al
	nop
	nop

	mov     al,03h                  ;Strobe new data value
	out     dx,al
	nop
	nop
	nop
	nop

	mov     al,02h                  ;Clear strobe
	out     dx,al

	push    cx                      ;wait for electronics to settle
	mov     cx,127
pause:  nop
	loop    pause
	pop     cx

;
; elevation must map 0 to +180 to 0 to 255.
; note that the program could supply -90 - 0 and 181 - 269, but 
; the Yaesu G-5400 rotor won't go that far.  
;
; I could internally adjust azimuth to
; adjust for the reversed directions, I'll leave that
; for the future.
;
; the equation for this is ((el*27)/19)
;
	mov     ax,cx
	cmp     ax,0
	jge     evnoclip                ;any setting less than zero is
	mov     ax,0                    ;clipped to zero.
evnoclip:
	add     ax,cs:elOffset          ;add the user's offset
	cmp     ax,180
	jle     evgood                  ;any setting greater-than 180
	mov     ax,180                  ;is clipped to 180
evgood:
	mov     bl,27
	mul     bl
	mov     bl,19
	div     bl

	mov     dx,cs:lptDataOut        ; move data into FODTRK hardware
	out     dx,al
	nop
	nop

	mov     dx,cs:lptCtrlOut        ; Select Elevation DAC
	mov     al,00h
	out     dx,al
	nop
	nop

	mov     al,01h                  ; Strobe the new value
	out     dx,al
	nop
	nop
	nop
	nop

	mov     al,00h                  ; clear the strobe
	out     dx,al

	pop     cx              ; retrieve elevation
	pop     cx              ; retrieve azimuth

	popf                    ; restore direction flag

st_exit:        iret
settarg endp

lastbyte        equ     this byte       ; flag for end of memory used
;*****************
; Everything past this point is thrown away
; after the main entry code tells DOS to
; terminate and stay resident.
; DO NOT put anything here that you need
; to process the KCT bios hook.
;******************
;
; calc_port -- setup i/o port addresses
;
; At Entry:
; AX = lpt base address
;
; At Exit:
;  Variables lptDataOut, lptCtrlIn, lptCtrlOut, lptControl initialized.
;

calc_port       proc near
	mov     bx,DATAOUT
	add     bx,ax
	mov     lptDataOut,bx
;
	mov     bx,CTRLIN
	add     bx,ax
	mov     lptCtrlIn,bx
;
	mov     bx,CTRLOUT
	add     bx,ax
	mov     lptCtrlOut,bx
;
	mov     bx,CONTROL
	add     bx,ax
	mov     lptControl,bx
;
	ret

calc_port       endp

;
; use_lpt1 - Setup I/O registers to use LPT1 as FODTRACK port.
;  This is installed as a callback from the command-line
;  tokenizer.
;
use_lpt1        proc near
	mov      ax,LPT1
	call    calc_port
	ret

use_lpt1        endp

;
; use_lpt2 - Setup I/O registers to use LPT2 as FODTRACK port.
;  This is installed as a callback from the command-line
;  tokenizer.
;
use_lpt2        proc near
	mov      ax,LPT2
	call    calc_port
	ret
use_lpt2        endp

; use_port - Setup I/O registers to use Port at specified address.
;
;  This is installed as a callback from the command-line
;  tokenizer.
;
; At Entry:
;       DI points to start of zero-terminated ascii string
;       representing parallel port base address.
;
; At Exit:
;   Parallel Port addresses are set up address is hexidecimal
;   base address of port.
;
; TO DO:
;   Add check at entry to verify a option exists.
;   Add check after xtoi() to verify all characters were converted.
;
use_port        proc near
	mov     si,di
	call    xtoi                    ; convert string to binary
	call    calc_port               ; setup I/O addresses.
	ret
use_port        endp

; azimuth - Store string as value of azimuth offset.
;
;  This is installed as a callback from the command-line
;  tokenizer.
;
; At Entry:
;       DI points to start of zero-terminated ascii string
;       representing azimuth offset in degrees.
;
; At Exit:
;       Azimuth offset is set to specified value.
;
; TO DO:
;   Add check at entry to verify a option exists.
;   Add check after atoi() to verify all characters were converted.
;   Add check after atoi() to verify converted value is in range.
;
azimuth proc near
	mov     si,di
	call    atoi                    ; Convert offset to signed value
	mov     azOffset,ax             ; store new value.
	ret
azimuth endp

; elevation - Store string as value of elevation offset.
;
;  This is installed as a callback from the command-line
;  tokenizer.
;
; At Entry:
;       DI points to start of zero-terminated ascii string
;       representing elevation offset in degrees.
;
; At Exit:
;       Elevation offset is set to specified value.
;
; TO DO:
;   Add check at entry to verify a option exists.
;   Add check after atoi() to verify all characters were converted.
;   Add check after atoi() to verify converted value is in range.
;
elevation       proc near
	mov     si,di
	call    atoi
	mov     elOffset,ax
	ret
elevation       endp

;
; optTable - Table specifing the valid command line options.
;
;  This table is passed to the command line parser in order
;  to indicate which command line options are valid, and
;  how to process them.
;
;  Each record contains two entries: a zero-terminated string
;  indicating the text of the command option, followed by
;  a word containing the address of the subroutine to call
;  when the option is recognized.
;
;  If the command-line option string is followed by a ':', 
;  then the parser will assume that a value for the option follows.  
;  Before calling the option handler, DI will be set
;  to the first character past the colon.  The option string
;  will be terminated by a zero.
;
optTable:
	db      '/LPT1', 0
	dw      use_lpt1
	db      '/LPT2', 0
	dw      use_lpt2
	db      '/PORT', 0
	dw      use_port
	db      '/AZ', 0
	dw      azimuth
	db      '/EL', 0
	dw      elevation
	db      0               ;<--This must be last

_TEXT   ends

	end     install
